@@ -23,6 +23,9 @@ mergeInto(LibraryManager.library, {
 #endif
 #if ASSERTIONS
     '$ERRNO_MESSAGES', '$ERRNO_CODES',
+#endif
+#if ASSERTIONS && !MINIMAL_RUNTIME
+    '$demangleAll',
 #endif
     ],
   $FS__postset: function() {
@@ -116,7 +119,7 @@ FS.staticInit();` +
     // paths
     //
     lookupPath: (path, opts = {}) => {
-      path = PATH_FS.resolve(FS.cwd(), path);
+      path = PATH_FS.resolve(path);
 
       if (!path) return { path: '', node: null };
 
@@ -130,8 +133,8 @@ FS.staticInit();` +
         throw new FS.ErrnoError({{{ cDefine('ELOOP') }}});
       }
 
-      // split the path
-      var parts = PATH.normalizeArray(path.split('/').filter((p) => !!p), false);
+      // split the absolute path
+      var parts = path.split('/').filter((p) => !!p);
 
       // start at the root
       var current = FS.root;
@@ -1090,7 +1093,7 @@ FS.staticInit();` +
         if (!(path in FS.readFiles)) {
           FS.readFiles[path] = 1;
 #if FS_DEBUG
-          err("FS.trackingDelegate error on read file: " + path);
+          dbg("FS.trackingDelegate error on read file: " + path);
 #endif
         }
       }
@@ -1908,9 +1911,7 @@ FS.staticInit();` +
     DB_STORE_NAME: 'FILE_DATA',
 
     // asynchronously saves a list of files to an IndexedDB. The DB will be created if not already existing.
-    saveFilesToDB: (paths, onload, onerror) => {
-      onload = onload || (() => {});
-      onerror = onerror || (() => {});
+    saveFilesToDB: (paths, onload = (() => {}), onerror = (() => {})) => {
       var indexedDB = FS.indexedDB();
       try {
         var openRequest = indexedDB.open(FS.DB_NAME(), FS.DB_VERSION);
@@ -1941,9 +1942,7 @@ FS.staticInit();` +
     },
 
     // asynchronously loads a file from IndexedDB.
-    loadFilesFromDB: (paths, onload, onerror) => {
-      onload = onload || (() => {});
-      onerror = onerror || (() => {});
+    loadFilesFromDB: (paths, onload = (() => {}), onerror = (() => {})) => {
       var indexedDB = FS.indexedDB();
       try {
         var openRequest = indexedDB.open(FS.DB_NAME(), FS.DB_VERSION);
