@@ -5,16 +5,15 @@
  */
 
 var LibraryStackTrace = {
-  $demangle__deps: ['$withStackSave'],
+#if DEMANGLE_SUPPORT
+  $demangle__deps: ['$withStackSave', '__cxa_demangle', 'free'],
+#endif
   $demangle: function(func) {
 #if DEMANGLE_SUPPORT
     // If demangle has failed before, stop demangling any further function names
     // This avoids an infinite recursion with malloc()->abort()->stackTrace()->demangle()->malloc()->...
     demangle.recursionGuard = (demangle.recursionGuard|0)+1;
     if (demangle.recursionGuard > 1) return func;
-#if ASSERTIONS
-    assert(___cxa_demangle);
-#endif
     return withStackSave(function() {
       try {
         var s = func;
