@@ -80,12 +80,15 @@ COMPILE_TIME_SETTINGS = {
 
     # This is legacy setting that we happen to handle very early on
     'RUNTIME_LINKED_LIBS',
-    # TODO: should not be here
-    'AUTO_ARCHIVE_INDEXES',
-    'DEFAULT_LIBRARY_FUNCS_TO_INCLUDE',
 }.union(PORTS_SETTINGS)
 
 
+# Settings that don't need to be externalized when serializing to json because they
+# are not used by the JS compiler.
+INTERNAL_SETTINGS = {
+    'SIDE_MODULE_IMPORTS',
+}
+
 user_settings: Dict[str, str] = {}
 
 
@@ -153,6 +156,14 @@ class SettingsManager:
   def dict(self):
     return self.attrs
 
+  def external_dict(self, skip_keys={}): # noqa
+    external_settings = {k: v for k, v in self.dict().items() if k not in INTERNAL_SETTINGS and k not in skip_keys}
+    # Only the names of the legacy settings are used by the JS compiler
+    # so we can reduce the size of serialized json by simplifying this
+    # otherwise complex value.
+    external_settings['LEGACY_SETTINGS'] = [l[0] for l in external_settings['LEGACY_SETTINGS']]
+    return external_settings
+
   def keys(self):
     return self.attrs.keys()
 
